# encoding=utf-8

import functools
import jambalaya as ja
import utils


class browse_list_proxy(object):
    def __init__(self, cr, uid):
        self.models = utils.get_database_registry(cr.dbname)
        self.cr = cr
        self.uid = uid

    def __call__(self, browse_list):
        return browse_list_proxy_impl(self, browse_list)


class browse_list_proxy_impl(object):
    def __init__(self, proxy, browse_list):
        self.proxy = proxy
        self.browse_list = browse_list

    def __getattr__(self, name):
        val = getattr(self.rmodel, name)
        if callable(val):
            return functools.partial(val, *self.args)

        return val

    @ja.hacks.singleton
    def rmodel(self):
        return self.proxy.models.get(
            self.model_name,
        )

    @ja.hacks.singleton
    def model_name(self):
        return self.browse_list[0]._name

    @ja.hacks.singleton
    def ids(self):
        return [r.id for r in self.browse_list]

    @ja.hacks.singleton
    def args(self):
        return [self.proxy.cr, self.proxy.uid, self.ids]



# encoding=utf-8

from broadcast import broadcast
from browse_list_proxy import browse_list_proxy
from decorators import (single, batch)
from session import Session

def shell():
    import IPython
    import sys

    session = Session(sys.argv[1:])
    models = session.models
    cr = session.cr
    conn = session.conn
    uid = session.uid
    rr = session.rr
    conf = session.conf

    from openerp import netsvc
    wf = netsvc.LocalService("workflow")

    IPython.embed()



# encoding=utf-8

import functools


class decorator(object):

    def __init__(self, func):
        self.func = func

    def __get__(self, obj, type=None):

        @functools.wraps(self.func)
        def _func(*args, **kwargs):
            method = functools.partial(self.func, obj)
            if '_ids' in obj.__dict__:
                call = self.call_v8
            else:
                call = self.call

            return call(method, *args, **kwargs)

        return _func

    def call(self, method, *args, **kwargs):
        return method(*args, **kwargs)

    def call_v8(self, method, *args, **kwargs):
        return method(*args, **kwargs)


class single(decorator):
    def call(self, method, cr, uid, ids, *args, **kwargs):
        if _is_iterable(ids):
            ids = ids[0] if ids else None
        return super(single, self).call(method, cr, uid, ids, *args, **kwargs)

    def call_v8(self, method, *args, **kwargs):
        obj = method.args[0]

        obj._ids = obj._ids[:1]
        return super(single, self).call(method, *args, **kwargs)


class batch(decorator):
    def call(self, method, cr, uid, ids, *args, **kwargs):
        if not _is_iterable(ids):
            ids = [ids]
        return super(batch, self).call(method, cr, uid, ids, *args, **kwargs)


def _is_iterable(val):
    try:
        return iter(val)
    except:
        return False
